#include <Servo.h>

#define A  0
#define B  1
#define C  2
#define D  3
#define E  4

Servo leftMotor;
Servo rightMotor;
Servo neckServo;

int aPressed;
int bPressed;
int cPressed;
int dPressed;

bool aWasPressed = true;
bool bWasPressed = false;
bool cWasPressed = false;
bool dWasPressed = false;

int buttonPressed;

const int trigPin = 2;
const int echoPin = 3;

const int inputD = 11;
const int inputC = 10;
const int inputB = 9;
const int inputA = 8;

const int leftLightSensor = A3;
const int rightLightSensor = A2;

int leftLightVal;
int rightLightVal;

long duration;
int distance;
int pos;
int distArray[5];
int obstacleDistanceMax = 15;

void setup() {
  Serial.begin(9600);

  leftMotor.attach(12);
  rightMotor.attach(13);
  neckServo.attach(4);

  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  pinMode(inputD, INPUT);
  pinMode(inputC, INPUT);
  pinMode(inputB, INPUT);
  pinMode(inputA, INPUT);

  halt();

  delay(3000); //Wait three second before starting to run
}

void loop() {
  buttonPressed = readRemote();

  switch (buttonPressed) {
    case A:
      halt();
      break;

    case B:
      wanderer();
      break;

    case C:
      followLight();
      break;

    case D:
      avoidLight();
      break;
  }
}

void forward() {
  leftMotor.write(180);
  rightMotor.write(0);
}

void backward() {
  leftMotor.write(0);
  rightMotor.write(180);
}

void halt() {
  leftMotor.write(90);
  rightMotor.write(90);
}

void turnRight() {
  leftMotor.write(180);
  rightMotor.write(180);
}

void turnLeft() {
  leftMotor.write(0);
  rightMotor.write(0);
}

void veerRight() {
  leftMotor.write(180);
  rightMotor.write(90);
}

void veerLeft() {
  leftMotor.write(90);
  rightMotor.write(0);
}

void wanderer() {
  forward();
  getDistance();
  if (distance <= obstacleDistanceMax) {
    obstacleDetected();
  }
}

long getDistance() {
  digitalWrite(trigPin, LOW);
  delayMicroseconds(5);

  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);
  distance = (duration / 2) / 29.1;

  Serial.print("Distance = ");
  Serial.print(distance);
  Serial.println(" cm");

  return (distance);
}

void obstacleDetected() {
  halt();
  delay(200);
  backward();
  delay(200);
  halt();
  delay(100);
  switch (scan()) {
    case 1:
      Serial.println("Direction Far Left");
      turnLeft();
      delay(600);
      break;
    case 2:
      Serial.println("Direction Slight Left");
      turnLeft();
      delay(350);
      break;
    case 3:
      Serial.println("Direction Forward");
      break;
    case 4:
      Serial.println("Direction Slight Right");
      turnRight();
      delay(350);
      break;
    case 5:
      Serial.println("Direction Far Right");
      turnRight();
      delay(600);
      break;
    default:
      break;
  }
}

int scan() {
  long kMax = 0;
  long maxDist = 0;
  int option;

  for (byte i = 0, pos = 20; pos <= 160; i++, pos += 35) {
    neckServo.write(pos);
    delay(500);
    distArray[i] = getDistance();
  }

  for (byte k = 0; k < 5; k++)
    if (distArray[k] > maxDist) {
      maxDist = distArray[k];
      kMax = k;
    }
  Serial.print("The longest past is ");
  Serial.print(maxDist);
  Serial.println("CM.");
  Serial.print("Position of highest distance in the array = ");
  option = kMax + 1;
  Serial.println(option);
  neckServo.write(90);

  return (option);
}

int readRemote() {
  aPressed = digitalRead(inputA);
  bPressed = digitalRead(inputB);
  cPressed = digitalRead(inputC);
  dPressed = digitalRead(inputD);

  if (aPressed == HIGH) return A;
  if (bPressed == HIGH) return B;
  if (cPressed == HIGH) return C;
  if (dPressed == HIGH) return D;
}

void followLight() {
  leftLightVal = analogRead(leftLightSensor);
  rightLightVal = analogRead(rightLightSensor);
  leftLightVal = map(leftLightVal, 0, 1023, 0, 100);
  rightLightVal = map(rightLightVal, 0, 1023, 0, 100);

  Serial.print("Left Light Value = ");
  Serial.println(leftLightVal);
  Serial.print("Right Light Value = ");
  Serial.println(rightLightVal);

  if (leftLightVal > rightLightVal+1){
    veerLeft();
  }else if (rightLightVal > leftLightVal+1){
    veerRight();
  }else{
    halt();
  }
}

void avoidLight() {
  leftLightVal = analogRead(leftLightSensor);
  rightLightVal = analogRead(rightLightSensor);
  leftLightVal = map(leftLightVal, 0, 1023, 0, 100);
  rightLightVal = map(rightLightVal, 0, 1023, 0, 100);

  Serial.print("Left Light Value = ");
  Serial.println(leftLightVal);
  Serial.print("Right Light Value = ");
  Serial.println(rightLightVal);

  if (leftLightVal > rightLightVal+1){
    veerRight();
  }else if (rightLightVal > leftLightVal+1){
    veerLeft();
  }else{
    halt();
  }
}
